
/***************************** Include Files *********************************/
#include "main.h"

vdma_context_t  context_;

int resetRead(XAxiVdma *InstancePtr)
{
//    XAxiVdma_ChannelStop(&AxiVdma.ReadChannel);
//    while (XAxiVdma_ChannelIsRunning(&AxiVdma.ReadChannel)) ;

   XAxiVdma_ChannelReset(&InstancePtr->ReadChannel);

   int Polls = RESET_POLL;

   while (Polls && XAxiVdma_ChannelResetNotDone(&InstancePtr->ReadChannel)) {
      --Polls;
   }

   if (!Polls) {
     return (XST_FAILURE);
   }

   return (XST_SUCCESS);
}

int resetWrite(XAxiVdma *InstancePtr)
{
//    XAxiVdma_ChannelStop(&AxiVdma.WriteChannel);
//    while (XAxiVdma_ChannelIsRunning(&AxiVdma.WriteChannel)) ;

   XAxiVdma_ChannelReset(&InstancePtr->WriteChannel);

   int Polls = RESET_POLL;

   while (Polls && XAxiVdma_ChannelResetNotDone(&InstancePtr->WriteChannel)) {
      --Polls;
   }

   if (!Polls) {
     return (XST_FAILURE);
   }

   return (XST_SUCCESS);
}


/*****************************************************************************/
/**
* This function performance a reset of the VDMA device and checks the device is
* coming out of reset or not.
*
* @param DeviceId is the DMA device id.
*
* @return
*     - XST_SUCCESS if channel reset is successful
*     - XST_FAILURE if channel reset fails.
*
* @note     None.
*
******************************************************************************/
int AxiVDMASelfTestExample(XAxiVdma *InstancePtr, u16 DeviceId)
{
   XAxiVdma_Config *Config;
   int Status = XST_SUCCESS;

   Config = XAxiVdma_LookupConfig(DeviceId);
   if (!Config) {
         return XST_FAILURE;
   }

   /* Initialize DMA engine */
   Status = XAxiVdma_CfgInitialize(InstancePtr, Config, Config->BaseAddress);
   if (Status != XST_SUCCESS) {
      return XST_FAILURE;
   }

   Status = XAxiVdma_Selftest(InstancePtr);
   if (Status != XST_SUCCESS) {
      return XST_FAILURE;
   }

   return Status;
}

/*****************************************************************************/
/**
*
* This function configures the read channel of the VDMA.
*
* @param h_res is the horizontal resolution.
* @param v_res is the vertical resolution
*
* @return
*     - XST_SUCCESS if successful.
*     - XST_FAILURE if unsuccessful.
*
* @note     None.
*
******************************************************************************/
int configureRead(uint16_t h_res, uint16_t v_res)
{
   XStatus status;
   context_.ReadCfg.HoriSizeInput = h_res * AxiVdma.ReadChannel.StreamWidth;
   context_.ReadCfg.VertSizeInput = v_res;
   context_.ReadCfg.Stride = context_.ReadCfg.HoriSizeInput;
   context_.ReadCfg.FrameDelay = 1;
   context_.ReadCfg.EnableCircularBuf = 1;
   context_.ReadCfg.EnableSync = 0;
   context_.ReadCfg.PointNum = 0;
   context_.ReadCfg.EnableFrameCounter = 0;
   context_.ReadCfg.FixedFrameStoreAddr = 0; //park it on 0 until we sync
   status = XAxiVdma_DmaConfig(&AxiVdma, XAXIVDMA_READ, &context_.ReadCfg);
   if (XST_SUCCESS != status)
   {
     return (XST_FAILURE);
   }
   uint32_t addr = MEM_BASE_R_ADDR;
   for (int iFrm=0; iFrm<AxiVdma.MaxNumFrames; ++iFrm) {
      context_.ReadCfg.FrameStoreStartAddr[iFrm] = addr;
      xil_printf("VDMA Frame %d Addr: 0x%08x\r\n", iFrm, addr);
      //memset((void*)addr,0,context_.ReadCfg.HoriSizeInput * context_.ReadCfg.VertSizeInput);
      addr += context_.ReadCfg.HoriSizeInput * context_.ReadCfg.VertSizeInput;
   }
   status = XAxiVdma_DmaSetBufferAddr(&AxiVdma, XAXIVDMA_READ, context_.ReadCfg.FrameStoreStartAddr);
   if (XST_SUCCESS != status)
   {
     return (XST_FAILURE);
   }

   //Clear errors in SR
   XAxiVdma_ClearChannelErrors(&AxiVdma.ReadChannel, XAXIVDMA_SR_ERR_ALL_MASK);
   //Enable read channel error and frame count interrupts
   XAxiVdma_IntrEnable(&AxiVdma, XAXIVDMA_IXR_ERROR_MASK, XAXIVDMA_READ);

   return (XST_SUCCESS);
}

/*****************************************************************************/
/**
*
* This function configures the write channel of the VDMA.
*
* @param h_res is the horizontal resolution.
* @param v_res is the vertical resolution
*
* @return
*     - XST_SUCCESS if successful.
*     - XST_FAILURE if unsuccessful.
*
* @note     None.
*
******************************************************************************/
int configureWrite(uint16_t h_res, uint16_t v_res)
{
   XAxiVdma_ClearDmaChannelErrors(&AxiVdma, XAXIVDMA_WRITE, XAXIVDMA_SR_ERR_ALL_MASK);

   XStatus status;
   context_.WriteCfg.HoriSizeInput = h_res * AxiVdma.WriteChannel.StreamWidth;
   context_.WriteCfg.VertSizeInput = v_res;
   context_.WriteCfg.Stride = context_.WriteCfg.HoriSizeInput;
   context_.WriteCfg.FrameDelay = 0;
   context_.WriteCfg.EnableCircularBuf = 1;
   context_.WriteCfg.EnableSync = 0; //Gen-Lock
   context_.WriteCfg.PointNum = 0;
   context_.WriteCfg.EnableFrameCounter = 0;
   context_.WriteCfg.FixedFrameStoreAddr = 0; //ignored, since we circle through buffers
   status = XAxiVdma_DmaConfig(&AxiVdma, XAXIVDMA_WRITE, &context_.WriteCfg);
   if (XST_SUCCESS != status)
   {
      return (XST_FAILURE);
   }
   uint32_t addr = MEM_BASE_ADDR;
   for (int iFrm=0; iFrm<AxiVdma.MaxNumFrames; ++iFrm) {
      context_.WriteCfg.FrameStoreStartAddr[iFrm] = addr;
      addr += context_.WriteCfg.HoriSizeInput * context_.WriteCfg.VertSizeInput;
   }
   status = XAxiVdma_DmaSetBufferAddr(&AxiVdma, XAXIVDMA_WRITE, context_.WriteCfg.FrameStoreStartAddr);
   if (XST_SUCCESS != status)
   {
      return (XST_FAILURE);
   }
   //Clear errors in SR
   XAxiVdma_ClearChannelErrors(&AxiVdma.WriteChannel, XAXIVDMA_SR_ERR_ALL_MASK);
   //Unmask error interrupts
   XAxiVdma_MaskS2MMErrIntr(&AxiVdma, ~XAXIVDMA_S2MM_IRQ_ERR_ALL_MASK, XAXIVDMA_WRITE);
   //Enable write channel error and frame count interrupts
   XAxiVdma_IntrEnable(&AxiVdma, XAXIVDMA_IXR_ERROR_MASK, XAXIVDMA_WRITE);

   return (XST_SUCCESS);
}
